"""Tests for @project@."""

import unittest
#from nose.tools import ok_, eq_, raises
#from nose import SkipTest
    
class @module.capitalize@TestCase(unittest.TestCase):
    def setUp(self):
        pass
        
    def tearDown(self):
        pass
        
    def test_@module@(self):
        # do something to test @module@
        pass